#!/usr/bin/env python
# encoding: utf-8
"""
befriender_client.py

This is the main executable for the befriender client.

Created by Elliot Pennington on 2008-11-30. 
Edited by Gregory Sheaffer
"""

from socket import *
from message import *
from Queue import *
from Tkinter import *
import tkMessageBox
import threading
import receiver, sender, gui
import sys
import net_info

host = "localhost"
port = 1465
buf_sz = 1024
addr = (host, port)

def connect_to_server():
    """ get a socket connection """
    sockfd = socket(AF_INET, SOCK_STREAM)
    sockfd.connect((net_info.addr, net_info.port))
    return sockfd

class BefrienderClient:
    
    """ 
    b_client represents an internal befriender cleint. A new instantiation
    of one of these is like a whole separate login. The flow of this class 
    will login based on a required username and password to the server, and
    then receive messages from the server. The b_client will listen and dispatch
    the messages to the appropriate window based on incoming screen name, or 
    spawn a new conversation thread to handle a new screen name. 
    @
    Attributes:
        sockfd: The socket descriptor for the connection to the server
        username: The username for the login of this client
        current_convos: List of screen names (strings) with which this user is currently in 
                    conversation
        recv_queues: dictionary association of Queues, each associated with a conversation.
                    The dictionary key is the screen name of the conversation partner.  
        recv_events: dictionary association of Events, each associated with a conversation.
                    The keys are the same as for the queues.
        receiver: The method to run for the receiver thread. In this case, the graphical window
                    thread for printing conversations and running a window.
        send_queue: The Queue for outgoing messages and commands. Popped only within the sender 
                    thread.
        send_event: The Event for the send queue and the sender thread. 
        sender: The thread created solely for the purpose of sending messages and commands
                    to the server.
    """
    
    def __init__(self):
        """ 
        This constructor will serve forever. Instantiating an object of this type
        is all you should ever need to do with it. 
        """
        global addr
        global buf_sz
        
        self.sockfd = connect_to_server()
  
        self.current_convos = []
        self.recv_events = {}
        self.recv_queues = {}
        self.receiver = gui.tabbed_frame
        self.outer_event = threading.Event()
        
        self.send_queue = Queue()           #note that the gui refers to these as outqueue and outevent
        self.send_event = threading.Event()
        self.sender = threading.Thread(target=sender.Sender,
                                       args=(self.sockfd, self.send_queue, self.send_event))
        self.sender.start()
        
        self.username = ""
    #TODO: set to 0 default, once database is running
        self.permission = 1 # 0 = normal user, 1 = admin
        self.loginwindow = gui.login_window
        gui.login_window(self.send_queue, self.send_event, self.set_username, self.set_logout2)
#To test login without window
#        self.send_queue.put("~server:login:%s:%s" % ("username", "password"), True)
#        self.send_event.set()
#        self.send_event.clear()

#Setup for the main tab window. This waits until the Tk() from the login window ends.
        self.root = Tk()        
        self.spawn_root_thread()
#Start waiting        
        self.wait_forever()
        
    def wait_forever(self):
        """ 
        Wait for messages as they come in from the server.
        
        If a message is from a new sender, make a new thread to handle
        that conversation. The new thread gets a new queue, created for IPC. 
        The new screen name is added to current_convos.
        """         
        while(1):
            print "Wait forever loop"
#receive message
            data = self.sockfd.recv(buf_sz)
#use MessageObject to decode and read message                 
            msg = MessageObject(data)
            my_sn = msg.getSn()
            my_msg = msg.getMsg()
            print ("Received message "+my_sn+":"+my_msg)
    #Test creating the window
            """
            self.recv_queues["DrizztDoUrden50"] = Queue()
            self.recv_events["DrizztDoUrden50"] = threading.Event()
            self.current_convos.append("DrizztDoUrden50")
            
            self.spawn_thread(self.win(), "DrizztDoUrden50", 
                              self.recv_queues["DrizztDoUrden50"], 
                              self.send_queue, 
                              self.recv_events["DrizztDoUrden50"], 
                              self.send_event)
         
            #make sure the gui is about done
            self.outer_event.wait()
            #snap control back to the gui for the mainloop to start
            self.outer_event.set()
            self.outer_event.clear()
            import time
            time.sleep(1)
            self.recv_queues["DrizztDoUrden50"].put("testmessage", True)
            
            #now we know we can set this and make the receiver write
            self.recv_events["(insert screenname for test here)"].set()
            self.recv_queues["(insert screenname for test here)"].join()
            self.recv_events["(insert screenname for test here)"].clear()
    #end test
            """
#if the message contains a client command            
            if my_sn=="~client":
                self.process_command(msg.getMsg[0], msg.getMsg()[1:])
#Otherwise it is a regular aim message            
            else: 
#if the message is not part of a current conversation, add it to the set of conversations and build a new conversation window tab
                if my_sn not in self.current_convos:
                    print ("New Conversation")
                    self.recv_queues[my_sn] = Queue()
                    self.recv_events[my_sn] = threading.Event()
                    self.current_convos.append(my_sn)
                    self.spawn_thread(self.win(), my_sn, 
                                  self.recv_queues[my_sn], 
                                  self.send_queue, 
                                  self.recv_events[my_sn], 
                                  self.send_event)
                    #make sure the gui is about done
                    #self.outer_event.wait()
                    #snap control back to the gui for the mainloop to start
                    self.outer_event.set()
                    self.outer_event.clear()
                    import time
                    time.sleep(1)
                    self.recv_queues[my_sn].put(my_msg, True)
                    #now we know we can set this and make the receiver write
                    self.recv_events[my_sn].set()
                    self.recv_queues[my_sn].join()
                    self.recv_events[my_sn].clear()
#Otherwise just place the message on the appropriate queue
                else:
                    self.recv_queues[my_sn].put(my_msg, True)
                    self.recv_events[my_sn].set()
                    #    self.recv_queues[my_sn].join()
                    self.recv_events[my_sn].clear()
    """ 
    Create a new thread and run process_request_thread in it    
    """
    def spawn_thread(self, master, sn, in_queue, out_queue, in_event, out_event):
        t = threading.Thread(target=self.process_request_thread,
                             args=(master, sn, in_queue, out_queue, in_event, out_event, self.outer_event))
        t.start()
    
    def process_request_thread(self, master, sn, in_queue, out_queue, in_event, out_event, top_event):
        """ Called from within the new thread. Runs the handler. """ 
        self.receiver = gui.tabbed_frame(self.win(), sn, in_queue, out_queue, in_event, out_event, top_event)
        self.win.add_screen(self.receiver, sn)
        
    def spawn_root_thread(self):
        """ Create a new thread to run the main window """
        th = threading.Thread(target=self.process_root_thread,args=())
        th.start()

    """ Command for UI close ('X') button, pops up a dialogue box for confirmation then logs out """
    def process_root_thread(self):
        """ Add the tab window to root and the menu/options windows, then start the mainloop """
        self.root.protocol("WM_DELETE_WINDOW", self.close_UI)
        self.root.title("Befriender Client")
        
        self.win = gui.Window(self.root)
        self.menu = gui.menu_window(self.win(), self.username, self.send_queue, self.send_event, self.set_logout)
        self.win.add_screen(self.menu, "Menu")
        self.options = gui.options_window(self.win(), self.username, self.permission, self.send_queue, self.send_event)
        self.win.add_screen(self.options, "Options")
        #test displaying history/note windows----------
        #self.history = gui.history_window(self.win(), "HISTORY BODY GOES HERE")
        #self.win.add_screen(self.history, "History")
        
        #self.notes = gui.note_window(self.win(), "NOTES:", "SCREENNAME", self.send_queue, self.send_event)
        #self.win.add_screen(self.notes, "Notes for SCREENNAME")
        #test -----------------------------------------
        self.root.mainloop()
    
    """
    Sets the bclients username to the given string
    
    @param username2: string containing the username
    """
    def set_username(self, username2):
        print "Username changed to: %s" % username2
        self.username = username2
    
    """
    Handles parsing and executing the various clientside commands.
    
    @param cmd: The string containing the command name ('logout', 'permission', 'history', or 'notes')
    @param body: List of strings containing the data needed for the command
    """
    def process_command(self, cmd, body):
        print "Processing client command: %s" % cmd
        if cmd=="logout":
            tkMessageBox.showinfo("Login", "The server has refused your login request, please try again.")
            self.set_logout2()
        if cmd=="permission":
            tkMessageBox.showinfo("Login", "Login successful!")
            self.permission=body[0]
        if cmd=="history":
            tkMessageBox.showinfo("History", "History record received!")
            self.history = gui.history_window(self.win(), body[0])
            self.win.add_screen(self.history, "History")
        if cmd=="notes":
            self.notes = gui.note_window(self.win(), body[0], body[1], self.send_queue, self.send_event)
            self.win.add_screen(self.notes, "Notes for %s" % body[1])
    
    """
    Close the root GUI and all connected windows, then end the program
    """    
    def set_logout(self):
        print "Logging out, closing GUI"
        self.root.destroy()        
        sys.exit(0)

    """ 
    As set_logout, but does not close the root gui.
    For closing the program from the login window, where the bclint gui has not been initialized yet. 
    """ 
    def set_logout2(self):
        print "Logging out, closing GUI"
        sys.exit(0)
    
    """ 
    Command for UI close ('X') button, pops up a dialogue box for confirmation, then calls set_logout
    """     
    def close_UI(self):
        if tkMessageBox.askokcancel("Quit", "Are you sure you want to log out?"):
            self.send_queue.put("~server:logout", True)
            self.send_event.set()
            self.send_event.clear()
            self.set_logout()
        
if __name__ == '__main__':
    client = BefrienderClient()
#   client.login("username", "password")
#   client.process_incoming_messages()
   #test 
#    data = "Screenname:F:F:T: UU,:F:A:en:<HTML>Hello</HTML>"
#    msg = deserialize(data)
#    print "Received: "
#    msg.display()
#    client.receive_stuff(msg)

"""
Basic server communication api:
~server:login:usr:pass
"History logs. Format is username:day:month:year
    0 in any field = wildcard search, return applicable history titles in same name/date format so user could pick a specific one
    all fields filled returns specific records if there are any
~server:settings:set:key:value #might want to make specific commands, like 'changepassword:username:password'
~server:settings:get:key
~server:logout
~server:available:[true|false]
~server:endConvo:sn

Server->client
~client:logout (failed login)
~client:permission:0/1 (return on sucessful login, 0=user 1=admin) 
~client:notes:string:sn  (note sheet form)
~client:history:string (history item for viewing)
"""