#!/usr/bin/env python
# encoding: utf-8
"""
gui.py

This is the graphical section for the client.

Created by Elliot Pennington on 2008-12-15.
Edited by Gregory Sheaffer
"""


from Queue import *
from Tkinter import *
import tkMessageBox
import threading
import receiver

class Window:
    
    """
    Tabbed window
    Let's see if this works
    """
    
    def __init__(self, master):
        self.active_fr = None
        self.count = 0
        self.choice = IntVar(0)
        self.side = LEFT
        
        self.rb_fr = Frame(master, borderwidth = 2, relief = RIDGE)
        self.rb_fr.pack(fill = BOTH)
        
        self.screen_fr = Frame(master, borderwidth = 2, relief = RIDGE)
        self.screen_fr.pack(fill = BOTH)
        
    def __call__(self):
        return self.screen_fr
    
    def add_screen(self, fr, title):
        #Create a tab for the frame with the given title and adds it to the window
        b = Radiobutton(self.rb_fr, text = title, indicatoron = 0, \
                        variable = self.choice, value = self.count, \
                        command = lambda: self.display(fr))
        b.pack(fill = BOTH, side = self.side)
        
        if not self.active_fr:
            fr.pack(fill = BOTH, expand = 1)
            self.active_fr = fr
        
        self.count += 1
        
        return b

    def display(self, fr):
        self.active_fr.forget()
        fr.pack(fill = BOTH, expand = 1)
        self.active_fr = fr

class tabbed_frame (Frame):
    """
    Special frame for tabbing in the Window class, holds a conversation
    """
    
    def __init__(self, master, sn, in_queue, out_queue, in_event, out_event, top_event):

        Frame.__init__(self, master)
        self.sn = sn
        self.in_queue = in_queue
        self.out_queue = out_queue 
        self.out_event = out_event
        self.in_event = in_event
        self.print_queue = Queue() 
        self.print_event = threading.Event()
        
        self.topframe = Frame(self)
        self.midframe = Frame(self)
        
        # Configure TOPFRAME
        Label (self, text='Conversation with: %s' % self.sn).pack(side=TOP,padx=10,pady=10)
    
        scroll = Scrollbar(self)
        scroll2 = Scrollbar(self)
        
        self.msg_txt = Text(self.topframe, padx = 10, pady = 10, width = 65, height = 20)
        self.msg_txt.configure(yscrollcommand = scroll.set)
        self.msg_txt.pack(side = LEFT)
        scroll.config(command=self.msg_txt.yview)
        scroll.pack(side = RIGHT, fill=Y)
        self.topframe.pack(side = TOP)
        self.msg_txt.configure(state=DISABLED)
        
        self.spawn_msg_frame(self.sn, self.in_queue, self.in_event)
        self.spawn_msg_frame("samariteen", self.print_queue, self.print_event)
     
        # Configure MIDFRAME
        self.entry_frame = Text(self.midframe, width = 60, height = 3)
        self.entry_frame.configure(yscrollcommand = scroll2.set)
        self.entry_frame.pack(side=TOP, padx = 10, pady = 10)
        scroll2.config(command=self.entry_frame.yview)
        scroll2.pack(side=RIGHT, fill=Y)
        self.midframe.pack(side = BOTTOM)
        self.entry_frame.focus_set()
        
        #Buttons        
        self.send = Button(self, text="Send", width=10, command=self.out)
        self.send.pack(side = RIGHT)
        
        self.end = Button(self, text="End Conversation", width=15, command=self.end_Conversation)
        self.end.pack(side = LEFT)
        
    def out(self):
        data = self.entry_frame.get("0.0", END)
        self.entry_frame.delete("0.0", END)
        
        self.print_queue.put("%s" % data, True)
        self.print_event.set()
        self.print_event.clear()
        
        self.out_queue.put("%s::en:%s" % (self.sn, data), True)
        self.out_event.set()
        self.out_event.clear()
    
    def end_Conversation(self):
        if tkMessageBox.askokcancel("End Conversation", "Are you sure you want to end the conversation?"):
            self.out_queue.put("~server:endConvo:%s" % self.sn, True) 
            self.out_event.set()
            self.out_event.clear()
# TODO: Open notes window, remove tab, remove from client conversation list
            self.destroy()
      		
    def spawn_msg_frame(self, sn, q, ev):
        t = threading.Thread(target=self.spawn_receiver,
                             args=(sn, q, ev, self.msg_txt.insert, self.msg_txt.configure))
        t.start()

    def spawn_receiver(self, sn, in_queue, in_event, insert, config):
        receiver.Receiver(sn, in_queue, in_event, insert, config)    

class menu_window(Frame):
    """
    A simple tab shown after login, containts the users username and buttons for changing IM availability and logging out.
    
    @param master: The master Tk frame to tie this window to.
    @param username: The username of the befriender logged in
    @param out_queue: The main client outqueue, for sending the login to the server
    @param out_event: The main client outevent, for sending the login to the server
    @param set_logout: The main client set_logout function, to close the befriender socket and application if the user closes this window without logging in
    """
    def __init__(self, master, username, out_queue, out_event, set_logout):
        Frame.__init__(self, master)
        self.frame = Frame(self)
        
        self.username = username
        self.out_queue = out_queue
        self.out_event = out_event
        self.set_logout = set_logout
        self.availability = 1
        
        Label (self, text= "Logged in as %s" % username).pack(side=TOP, padx=10, pady=10)
        
        self.avail_label = Label(self.frame, text="Available for conversations", fg="darkgreen")
        self.avail_label.pack(side=TOP)     
        
        self.availability_button = Button(self.frame, text="Toggle AIM Availability", width=20, command=self.change_availability)
        self.availability_button.pack(side = TOP, padx=10, pady=10)
        
        self.logout_button = Button(self.frame, text="Logout", width=20, command=self.logout)
        self.logout_button.pack(side = TOP, padx=10, pady=10)
        
        self.frame.pack()
        
    def change_availability(self):
        # Toggles the user's availability for new conversations with the server
        if(self.availability == 1):
            #Set availability to false
            self.out_queue.put("~server:available", True)
            self.out_event.set()
            self.out_event.clear()
            self.avail_label.configure(text = "Unavailable for conversations", fg="red") 
            self.availability = 0
            print "Menu: Availability toggled off"
        else:
            #Set availability to true
            self.out_queue.put("~server:available", True)
            self.out_event.set()
            self.out_event.clear()
            self.avail_label.configure(text= "Available for conversations", fg="darkgreen")
            self.availability = 1
            print "Menu: Availability toggled on"
    
    def logout(self):
        # Prompts with a dialogue box, then sends logout string and closes GUI and program
        if tkMessageBox.askokcancel("Quit", "Are you sure you want to log out?"):
            self.out_queue.put("~server:logout", True)
            self.out_event.set()
            self.out_event.clear()
            self.set_logout()
            self.destroy()

#TODO:
class options_window(Frame):
    """
    This window allows the user to modify various options, depending on their permissions level 
    If the user has normal permissions has buttons for: Changing password, History searching
    If the user has admin permissions has buttons for: Welcome/busy/end conversation messages, 
       changing the notes form, adding a user, changing a users password, and searching 
       the history database
    
    @param master: The master Tk frame to tie this window to.
    @param permission: The permission level of the user as an integer. 1=admin, 0=user, other=invalid
    @param username: The username of the befriender using the options
    @param out_queue: The main client outqueue, for sending the login to the server
    @param out_event: The main client outevent, for sending the login to the server
    """
    def __init__(self, master, username, permission, out_queue, out_event):
        
        Frame.__init__(self, master)
        self.frame = Frame(self)
        self.out_queue = out_queue 
        self.out_event = out_event
        self.permission = permission
#Top label
        self.permissiontype = "user"
        if(self.permission == 1):
            self.permissiontype = "admin"
        Label (self.frame ,text= "Options for %s %s" % (self.permissiontype, username)).pack(side=TOP, padx=10, pady=10)
#Permission denial to non-users/admins
        if (self.permission != 1 and self.permission != 0):
            Label (self.frame ,text= "Error: invalid permissions").pack(side=TOP, padx=10, pady=10)
#Password change - user only      
        if (self.permission == 0):
            Label (self.frame, text = "Change Password. Enter desired password, enter it again for confirmation, then press submit.").pack(side=TOP)
            self.pwd_entry = Entry(self.frame, width = 20, show="*")
            self.pwd_entry.pack(side=TOP,padx = 10, pady = 3)
            self.pwd_confirm = Entry(self.frame, width = 20, show="*")
            self.pwd_confirm.pack(side=TOP,padx = 10, pady = 3)
            self.pass_button = Button(self.frame, text="Password Submit", width=20, command=self.password_button)
            self.pass_button.pack(side = TOP, padx=10, pady=10)
#Password change - admin only        
        if (self.permission == 1):
            Label (self.frame, text = "Change Password.\n Enter desired username, the password twice for confirmation, and press submit.").pack(side=TOP)
            self.user_pwd_entry = Entry(self.frame, width = 20, show="*")
            self.user_pwd_entry.pack(side=TOP,padx = 10, pady = 3)
            self.pwd_entry = Entry(self.frame, width = 20, show="*")
            self.pwd_entry.pack(side=TOP,padx = 10, pady = 3)
            self.pwd_confirm = Entry(self.frame, width = 20, show="*")
            self.pwd_confirm.pack(side=TOP,padx = 10, pady = 3)
            self.pass_button2 = Button(self.frame, text="Password Submit", width=20, command=self.password_button2)
            self.pass_button2.pack(side = TOP, padx=10, pady=10)                  

#Create user - admin only
        if (self.permission == 1):
            Label (self.frame, text = "Create User. \n Enter desired username, password, and permission level (1 for admin, 0 for user).").pack(side=TOP)
            self.user_name_entry = Entry(self.frame, width = 20, show="*")
            self.user_name_entry.pack(side=TOP,padx = 10, pady = 3)
            self.user_pass_entry = Entry(self.frame, width = 20, show="*")
            self.user_pass_entry.pack(side=TOP,padx = 10, pady = 3)
            self.user_perm_entry = Entry(self.frame, width = 20)
            self.user_perm_entry.pack(side=TOP,padx = 10, pady = 3)
            self.pass_button = Button(self.frame, text="Create User", width=20, command=self.user_button)
            self.pass_button.pack(side = TOP, padx=10, pady=10)

#Standardized message options - admin only        
        if (self.permission == 1):
            Label (self.frame, text = "Change server Welcome/Busy/Goodbye messages and standard note form. \n Enter desired message, then press associated button.").pack(side=TOP)
            self.message_entry = Entry(self.frame, width = 30)
            self.message_entry.pack(side=TOP,padx = 10, pady = 3)
            self.wlcm_button = Button(self.frame, text="Welcome Message", width=20, command=self.welcome_button)
            self.wlcm_button.pack(side = TOP, padx=10, pady=10)
            self.bsy_button = Button(self.frame, text="Busy Message", width=20, command=self.busy_button)
            self.bsy_button.pack(side = TOP, padx=10, pady=10)
            self.gby_button = Button(self.frame, text="Goodbye Message", width=20, command=self.goodbye_button)
            self.gby_button.pack(side = TOP, padx=10, pady=10)
            self.nte_button = Button(self.frame, text="Note Form", width=20, command=self.note_button)
            self.nte_button.pack(side = TOP, padx=10, pady=10)
#history search - users and admins (search permissions handled serverside)    
        if (self.permission == 1 or self.permission ==0):
            Label (self.frame, text = "History logs. \n Format is [username:day:month:year] (no brackets). \n Enter 0 for unknown fields to search, fill all fields to display specific records.").pack(side=TOP)
            self.history_entry = Entry(self.frame, width = 30)
            self.history_entry.pack(side=TOP,padx = 10, pady = 3)
            self.pass_button = Button(self.frame, text="History request", width=20, command=self.history_button)
            self.pass_button.pack(side = TOP, padx=10, pady=10)

        self.frame.pack()
        
#Need to change 'key' field for all of these to correct value, or make commands
    def password_button(self):
        print "Changing password"
        pass1 = self.pwd_entry.get()
        self.pwd_entry.delete(0, END)
        pass2 = self.pwd_confirm.get()
        self.pwd_confirm.delete(0, END) 
        if(pass1 == pass2):
                    #TODO fix
            self.out_queue.put("~server:settings:set:key:%s" % (pass1), True)
            self.out_event.set()
            self.out_event.clear()
            tkMessageBox.showinfo("Password", "Password change request sent!")
        else:
            print "Password and confirmation did not match, not sent."
            tkMessageBox.showwarning("Password", "Error: Password and Confirmation did not match")
    
    def password_button2(self):
        print "Changing password"
        usr1 = self.user_pwd_entry.get()
        self.user_pwd_entry.delete(0, END)
        pass1 = self.pwd_entry.get()
        self.pwd_entry.delete(0, END)
        pass2 = self.pwd_confirm.get()
        self.pwd_confirm.delete(0, END) 
        if(pass1 == pass2):
                                #TODO fix
            self.out_queue.put("~server:settings:set:key:%s:%s" % (pass1, usr1), True)
            self.out_event.set()
            self.out_event.clear()
            tkMessageBox.showinfo("Password", "Password change request sent!")
        else:
            print "Password and confirmation did not match, not sent."
            tkMessageBox.showwarning("Password", "Error: Password and Confirmation did not match")

    
    def welcome_button(self):
        print "Changing welcome message"
        data = self.message_entry.get()
        self.message_entry.delete(0, END)
                            #TODO fix
        self.out_queue.put("~server:settings:set:key:%s" % (data), True)
        self.out_event.set()
        self.out_event.clear()
        tkMessageBox.showinfo("Notice", "Welcome message change request sent!")

    def busy_button(self):
        print "Changing busy message"
        data = self.message_entry.get()
        self.message_entry.delete(0, END)
                            #TODO fix
        self.out_queue.put("~server:settings:set:key:%s" % (data), True)
        self.out_event.set()
        self.out_event.clear()
        tkMessageBox.showinfo("Notice", "Busy message change request sent!")

    def goodbye_button(self):
        print "Changing goodbye message"
        data = self.message_entry.get()
        self.message_entry.delete(0, END)
                            #TODO fix
        self.out_queue.put("~server:settings:set:key:%s" % (data), True)
        self.out_event.set()
        self.out_event.clear()
        tkMessageBox.showinfo("Notice", "Goodbye message change request sent!")

    def note_button(self):
        print "Changing note form"
        data = self.message_entry.get()
        self.message_entry.delete(0, END)
                            #TODO fix
        self.out_queue.put("~server:settings:set:key:%s" % (data), True)
        self.out_event.set()
        self.out_event.clear()
        tkMessageBox.showinfo("Notice", "Note form change request sent!")
        
    def history_button(self):
        print "History search sent"
        data = self.history_entry.get()
        self.history_entry.delete(0, END)
        self.out_queue.put("~server:history:%s" % (data), True)
        self.out_event.set()
        self.out_event.clear()
        
    def user_button(self):
        print "User creation request sent"
        usrnm = self.user_name_entry.get()
        self.user_name_entry.delete(0, END)
        usrpas = self.user_pass_entry.get()
        self.user_pass_entry.delete(0, END)
        usrper = self.user_perm_entry.get()
        self.user_perm_entry.delete(0, END)
        if(usrnm == "" or usrpas == "" or usrper == ""):
            print "User create field blank, not sent."
            tkMessageBox.showwarning("Create User", "Error: fields not complete")
        else:
            #TODO fix
            self.out_queue.put("~server:settings:set:key:%s:%s" % (pass1, usr1), True)
            self.out_event.set()
            self.out_event.clear()
            tkMessageBox.showinfo("Create User", "User creation request sent!")
        
class login_window:    
    """
    A window to collect the login user name and password and send it to the server as a login command for authentication before closing. 
    Note that this uses a root Tk() window, and cannot be run with other root tk windows concurrently in the same application on windows. 
    
    @param out_queue: The main client outqueue, for sending the login to the server
    @param out_event: The main client outevent, for sending the login to the server
    @param set_username: The main client set_username function, for setting the client username to the entered value
    @param set_logout: The main client set_logout function, to close the befriender socket and application if the user closes this window without logging in
    """
    
    def __init__(self, out_queue, out_event, set_username, set_logout):
        
        print "Login window"
        
        self.root=Tk()
        self.root.protocol("WM_DELETE_WINDOW", self.close)
        self.root.title("Befriender Login")
        
        self.frame = Frame(self.root)
        self.out_queue = out_queue
        self.out_event = out_event
        self.set_username = set_username
        self.set_logout = set_logout
        
        Label (self.frame, text='Login').pack(side=TOP,padx=15,pady=10)
        Label (self.frame, text='Enter your Username & Password').pack(side=TOP)
        
        self.usr_entry = Entry(self.frame, width = 20)
        self.usr_entry.pack(side=TOP, padx = 10, pady = 3)
        self.usr_entry.focus_set()
        
        self.pwd_entry = Entry(self.frame, width = 20, show="*")
        self.pwd_entry.pack(side=TOP,padx = 10, pady = 3)

        self.login = Button(self.frame, text="Login", width=10, command=self.out)
        self.login.pack(side = RIGHT, pady=5)
        self.frame.pack()
        self.root.mainloop()
    
    def out(self):
        username = self.usr_entry.get()
        password = self.pwd_entry.get()
#        print username
#        print password
        self.out_queue.put("~server:login:%s:%s" % (username, password), True)
        self.out_event.set()
        self.out_event.clear()
        self.set_username(username)
        self.root.destroy()
        
    def close(self):
        if tkMessageBox.askokcancel("Quit", "Are you sure you want to quit before logging in?"):
            self.set_logout()
            
class history_window(Frame):
    """
    Simple tab for displaying a history log in a scrollable text frame. Contains button for closing tab.
    
    @param master: The master Tk frame to tie this window to.
    @param history: The history to display in the text frame, as a string 
    """
    def __init__(self, master, history):
        Frame.__init__(self, master)
        self.frame = Frame(self)
        self.history = history
#Title
        Label (self, text= "History").pack(side=TOP, padx=10, pady=10)
#Text frame        
        scroll = Scrollbar(self)
        
        self.history_txt = Text(self.frame, padx = 10, pady = 10, width = 65, height = 50)
        self.history_txt.configure(yscrollcommand = scroll.set)
        self.history_txt.pack(side = LEFT)
        
        scroll.config(command=self.history_txt.yview)
        scroll.pack(side = RIGHT, fill=Y)
#Insert history, then disable further editing        
        self.history_txt.insert("0.0", self.history)
        self.history_txt.configure(state=DISABLED)
#Button        
        self.close_button = Button(self.frame, text="Close", width=10, command=self.close_button)

        self.close_button.pack(side = BOTTOM, padx=10, pady=10)
        self.frame.pack()

    def close_button(self):
        self.destroy()

class note_window(Frame):
    """
    Simple tab for displaying a note entry form in an editable text frame. Contains button for submitting the form and then closing the tab.
    
    @param master: The master Tk frame to tie this window to.
    @param notes: The notes to display in the text frame, as a string    
    @param sn: The screen name the notes will be associated with, as a string
    @param out_queue: The main client outqueue, for sending the notes form to the server
    @param out_event: The main client outevent, for sending the notes form to the server
    """
    
    def __init__(self, master, notes, sn, out_queue, out_event):
        Frame.__init__(self, master)
        self.frame = Frame(self)
        self.notes = notes
        self.out_queue=out_queue
        self.out_event=out_event
        self.sn=sn
#Title
        Label (self, text= "Notes for "+sn).pack(side=TOP, padx=10, pady=10)
#Text frame        
        scroll = Scrollbar(self)
        
        self.note_txt = Text(self.frame, padx = 10, pady = 10, width = 65, height = 50)
        self.note_txt.configure(yscrollcommand = scroll.set)
        self.note_txt.pack(side = LEFT)
        
        scroll.config(command=self.note_txt.yview)
        scroll.pack(side = RIGHT, fill=Y)
#Insert notes        
        self.note_txt.insert("0.0", self.notes)
#Button        
        self.send_button = Button(self.frame, text="Send", width=10, command=self.send_button)
        self.send_button.pack(side = BOTTOM, padx=10, pady=10)
        self.frame.pack()

    def send_button(self):
        self.out_queue.put("~server:notes:%s:%s" % (self.note_txt.get("0.0", END), self.sn), True)
        self.out_event.set()
        self.out_event.clear()
        self.destroy()



"""
TKinter example

a = Tk()
n = Window(a)

# uses the notebook's frame
f1 = Frame(n())
b1 = Button(f1, text="Button 1")
e1 = Entry(f1)
# pack your widgets before adding the frame 
# to the notebook (but not the frame itself)!
b1.pack(fill=BOTH, expand=1)
e1.pack(fill=BOTH, expand=1)

f2 = Frame(n())
# this button destroys the 1st screen radiobutton
b2 = Button(f2, text='Button 2', command=lambda:x1.destroy())
b3 = Button(f2, text='Beep 2', command=lambda:n.add_screen(f2, "lol"))
b2.pack(fill=BOTH, expand=1)
b3.pack(fill=BOTH, expand=1)

f3 = Frame(n())

# keeps the reference to the radiobutton (optional)
x1 = n.add_screen(f1, "Screen 1")
n.add_screen(f2, "Screen 2")
n.add_screen(f3, "dummy")

if __name__ == "__main__":
    a.mainloop()

"""

"""
Depreciated individual window conversation gui, replaced by tabbed frame    
class c_window:
    
    def __init__(self, sn, in_queue, out_queue, in_event, out_event, top_event):
#       Set up window for conversation

        self.sn = sn
        self.in_queue = in_queue
        self.out_queue = out_queue 
        self.out_event = out_event
        self.in_event = in_event
        
        self.print_queue = Queue() 
        self.print_event = threading.Event()
        
        self.root2 = Tk()
        self.root2.withdraw()
        self.root = Toplevel(self.root2)
        self.root.protocol("WM_DELETE_WINDOW", self.end_Conversation)
        self.root.title("Conversation with %s" % self.sn)

        self.frame = Frame(self.root)
        
        #frames to hopefully fix scrollbar packing  
        self.topframe = Frame(self.frame)
        self.midframe = Frame(self.frame)
        
        Label (self.frame, text='Conversation with: %s' % self.sn).pack(side=TOP,padx=10,pady=10)
    
        scroll = Scrollbar(self.frame)
        scroll2 = Scrollbar(self.frame)
#topframe
        self.msg_txt = Text(self.topframe, padx = 10, pady = 10, width = 65, height = 20)
        self.msg_txt.configure(yscrollcommand = scroll.set)
        self.msg_txt.pack(side = LEFT)
        scroll.config(command=self.msg_txt.yview)
        scroll.pack(side = RIGHT, fill=Y)
        self.topframe.pack(side = TOP)
        self.msg_txt.configure(state=DISABLED)
        
        self.spawn_msg_frame(self.sn, self.in_queue, self.in_event)
        self.spawn_msg_frame("samariteen", self.print_queue, self.print_event)
#midframe        
        self.entry_frame = Text(self.midframe, width = 60, height = 3)
        self.entry_frame.configure(yscrollcommand = scroll2.set)
        self.entry_frame.pack(side=TOP, padx = 10, pady = 10)
        scroll2.config(command=self.entry_frame.yview)
        scroll2.pack(side=RIGHT, fill=Y)
        self.midframe.pack(side = BOTTOM)
        self.entry_frame.focus_set()
#Buttons        
        self.send = Button(self.frame, text="Send", width=10, command=self.out)
        self.send.pack(side = RIGHT)
        
        self.end = Button(self.frame, text="End Conversation", width=15, command=self.end_Conversation)
        self.end.pack(side = LEFT)
        
        self.frame.pack()
        print ("New GUI for conversation with %s" %sn)
        top_event.set()
        top_event.clear()
        top_event.wait()
        self.root.mainloop()
        
    def out(self):
        data = self.entry_frame.get("0.0", END)
        self.entry_frame.delete("0.0", END)
        
        self.print_queue.put("%s" % data, True)
        self.print_event.set()
        self.print_event.clear()
        
        self.out_queue.put("%s::en:%s" % (self.sn, data), True)
        self.out_event.set()
        self.out_event.clear()
    
    def end_Conversation(self):
        if tkMessageBox.askokcancel("End Conversation", "Are you sure you want to end the conversation?"):
            self.out_queue.put("~server:endConvo:%s" % self.sn, True) 
            self.out_event.set()
            self.out_event.clear()
# TODO: Open notes window, remove from current conversation list
            self.root2.destroy()
      
    def spawn_msg_frame(self, sn, q, ev):
        t = threading.Thread(target=self.spawn_receiver,
                             args=(sn, q, ev, self.msg_txt.insert, self.msg_txt.configure))
        t.start()


    def spawn_receiver(self, sn, in_queue, in_event, insert, config):
        receiver.Receiver(sn, in_queue, in_event, insert, config)
"""
